#include <qapplication.h>
#include <cstdlib>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "libmyth/mythcontext.h"
#include "libmyth/settings.h"
#include "libmyth/langsettings.h"
#include "libmyth/mythdbcon.h"
#include "libmyth/exitcodes.h"
#include "libmyth/compat.h"

extern "C" {
#include "avcodec.h"
#include "avformat.h"
}

// myth
#include "mythcontext.h"
#include "mythdbcon.h"
#include "lcddevice.h"
#include "tv.h"
#include "programinfo.h"
#include "uitypes.h"
#include "compat.h"
#include "upnp.h"
#include "upnpcmgr.h"

void initKeys(void)
{
    REG_KEY("Welcome", "STARTXTERM", "Open an Xterm window", "F12");
    REG_KEY("Welcome", "SHOWSETTINGS", "Show Mythshutdown settings", "F11");
    REG_KEY("Welcome", "STARTSETUP", "Start Mythtv-Setup", "");
}

int main(int argc, char **argv)
{
    QApplication a(argc, argv);
    gContext = NULL;
    gContext = new MythContext(MYTH_BINARY_VERSION);

	if (!gContext->Init()) 
    {
        VERBOSE(VB_IMPORTANT, "mythwelcome: Could not initialize myth context. "
                        "Exiting.");
        return FRONTEND_EXIT_NO_MYTHCONTEXT;
    }

    if (!MSqlQuery::testDBConnection())
    {
        VERBOSE(VB_IMPORTANT, "mythwelcome: Could not open the database. "
                        "Exiting.");
        return -1;
    }

    // Check command line arguments
    for (int argpos = 1; argpos < a.argc(); ++argpos)
    {
        if (!strcmp(a.argv()[argpos],"-v") ||
            !strcmp(a.argv()[argpos],"--verbose"))
        {
            if (a.argc()-1 > argpos)
            {
                if (parse_verbose_arg(a.argv()[argpos+1]) ==
                        GENERIC_EXIT_INVALID_CMDLINE)
                    return FRONTEND_EXIT_INVALID_CMDLINE;

                ++argpos;
            } 
            else
            {
                cerr << "Missing argument to -v/--verbose option\n";
                return FRONTEND_EXIT_INVALID_CMDLINE;
            }
        }
        else
        {
            cerr << "Invalid argument: " << a.argv()[argpos] << endl <<
                    "Valid options are: " << endl <<
                    "-v or --verbose debug-level    Use '-v help' for level info" << endl;
            return FRONTEND_EXIT_INVALID_CMDLINE;
        }
    }

    LanguageSettings::load("mythfrontend");

    gContext->LoadQtConfig();

    MythMainWindow *mainWindow = GetMythMainWindow();
    gContext->SetMainWindow(mainWindow);

    initKeys();

	/////////////////////////////////////////////////////////////////

	UPnp upnp;
	
	//libmythtv - boom!
	//ProgramInfo pi;

	/////////////////////////////////////////////////////////////////

	//delete gContext;

    return 0;
}
